from dacite import from_dict

from pokemontcgsdkasync.asyncclientcontext import AsyncClientContext


class QueryBuilder:

    def __init__(self, resource_type, transform=None):
        self.params = {}
        self.type = resource_type
        self.transform = transform

    async def find(self, resource_id: str):
        """Get a resource by its id
        
        Args:
            resource_id (string): Resource id
        Returns:
            object: Instance of the resource type
        """
        url = "{}/{}".format(self.type.RESOURCE, resource_id)
        response = (await AsyncClientContext.get(url))['data']

        # Transform json keys into names that are safe for python properties
        if self.transform:
            response = self.transform(response)

        return from_dict(self.type, response)

    async def where(self, **kwargs):
        """Adds a parameter to the dictionary of query parameters
        
        Args:
            **kwargs: Arbitrary keyword arguments.
        Returns:
            list of object: List of resource objects
        """
        for key, value in kwargs.items():
            self.params[key] = value

        return await self.all()

    async def all(self):
        """Get all resources, automatically paging through data

        Returns:
            list of object: List of resource objects
        """

        result_list = []
        fetch_all = True
        url = self.type.RESOURCE

        if 'page' in self.params:
            fetch_all = False
        else:
            self.params['page'] = 1

        while True:
            response = (await AsyncClientContext.get(url, self.params))['data']
            if len(response) > 0:
                if self.transform:
                    response = [self.transform(i) for i in response]

                result_list.extend([from_dict(self.type, item) for item in response])

                if fetch_all:
                    self.params['page'] += 1
                else:
                    break
            else:
                break

        return result_list

    async def array(self):
        """Get all resources and return the result as an array

        Returns:
            array of str: Array of resources
        """
        url = self.type.RESOURCE
        return (await AsyncClientContext.get(url, self.params))['data']



import unittest
from pokemontcgsdkasync import PokemonTcgException


class TestPokemonTcgException(unittest.TestCase):

    def test_constructor_sets_description(self):
        description = "An error has occurred"
        exception = PokemonTcgException(description)

        self.assertEqual(description, exception.__str__())



import unittest
from typing import List

from pokemontcgsdkasync import Subtype, AsyncClientContext


class TestSubtype(unittest.IsolatedAsyncioTestCase):

    async def test_all_returns_subtypes(self):
        async with AsyncClientContext():
            subtypes: List[Subtype] = await Subtype.all()

            self.assertTrue(len(subtypes) > 15)
            self.assertTrue('MEGA' in subtypes)



import unittest
from typing import List

from pokemontcgsdkasync import Card, AsyncClientContext


class TestCard(unittest.IsolatedAsyncioTestCase):

    async def test_find_returns_card(self):
        async with AsyncClientContext():
            card: Card = await Card.find('xy7-54')

            self.assertEqual('xy7-54', card.id)
            self.assertEqual('Gardevoir', card.name)
            self.assertEqual('Pokémon', card.supertype)
            self.assertEqual(['Stage 2'], card.subtypes)
            self.assertEqual('130', card.hp)
            self.assertEqual(['Fairy'], card.types)
            self.assertEqual('Kirlia', card.evolvesFrom)
            self.assertTrue(len(card.abilities) == 1)
            self.assertTrue(len(card.attacks) == 1)
            self.assertTrue(len(card.weaknesses) == 1)
            self.assertTrue(len(card.resistances) == 1)
            self.assertEqual(['Colorless', 'Colorless'], card.retreatCost)
            self.assertEqual(2, card.convertedRetreatCost)
            self.assertEqual('xy7', card.set.id)
            self.assertEqual('54', card.number)
            self.assertEqual('TOKIYA', card.artist)
            self.assertEqual('Rare Holo', card.rarity)
            self.assertEqual(
                'It has the power to predict the future. Its power peaks when it is protecting its Trainer.',
                card.flavorText)
            self.assertEqual([282], card.nationalPokedexNumbers)
            self.assertEqual('https://prices.pokemontcg.io/tcgplayer/xy7-54', card.tcgplayer.url)

    async def test_all_with_params_return_cards(self):
        async with AsyncClientContext():
            cards: List[Card] = await Card.where(q='supertype:pokemon subtypes:mega')
            self.assertTrue(len(cards) >= 70)

    async def test_all_with_page_returns_cards(self):
        async with AsyncClientContext():
            cards: List[Card] = await Card.where(page=1)
            self.assertEqual(250, len(cards))

    async def test_all_with_page_and_page_size_returns_card(self):
        async with AsyncClientContext():
            cards: List[Card] = await Card.where(page=1, pageSize=1)
            self.assertEqual(1, len(cards))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of pokemontcgsdk.
# https://github.com/Pole458/pokemon-tcg-sdk-python-async

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>
# Copyright (c) 2022, Paolo D'Alessandro <pole.gamedev@@gmail.com>

from pokemontcgsdkasync.card import Card
from pokemontcgsdkasync.set import Set
from pokemontcgsdkasync.subtype import Subtype
from pokemontcgsdkasync.supertype import Supertype
from pokemontcgsdkasync.type import Type
from pokemontcgsdkasync.rarity import Rarity
from pokemontcgsdkasync.asyncclientcontext import AsyncClientContext
from pokemontcgsdkasync.asyncclientcontext import PokemonTcgException
from pokemontcgsdkasync.querybuilder import QueryBuilder



from typing import Dict

from aiohttp import ClientSession
from aiohttp.abc import HTTPException

from pokemontcgsdkasync.pokemontcgexception import PokemonTcgException


class AsyncClientContext:

    _session: ClientSession = None

    def __init__(self, api_key: str = None):
        self.api_key: str = api_key
        self.headers: Dict = {'User-Agent': 'Mozilla/5.0'}
        if api_key is not None:
            self.headers['X-Api-Key'] = api_key

    async def __aenter__(self):
        AsyncClientContext._session = ClientSession(
            base_url="https://api.pokemontcg.io",
            headers=self.headers
        )
        return self

    async def __aexit__(self, *err):
        await AsyncClientContext._session.close()
        AsyncClientContext._session = None

    @classmethod
    async def get(cls, url: str, params: Dict = None):
        """Invoke an HTTP GET request on a url
        
        Args:
            url (string): URL endpoint to request
            params (dict): Dictionary of url parameters 
        Returns:
            dict: JSON response as a dictionary
        """
        try:
            async with cls._session.get("/v2/" + url, params=params) as resp:
                response = await resp.json()
                return response

        except HTTPException as err:
            raise PokemonTcgException(err.read())



import unittest
from typing import List

from pokemontcgsdkasync import Type, AsyncClientContext


class TestType(unittest.IsolatedAsyncioTestCase):

    async def test_all_returns_types(self):
        async with AsyncClientContext():
            types: List[Type] = await Type.all()

            self.assertEqual(
                ["Colorless", "Darkness", "Dragon", "Fairy", "Fighting", "Fire", "Grass", "Lightning", "Metal",
                 "Psychic", "Water"], types)



from dataclasses import dataclass
from typing import Optional


@dataclass
class TCGPrice:
    low: Optional[float]
    mid: Optional[float]
    high: Optional[float]
    market: Optional[float]
    directLow: Optional[float]


@dataclass
class TCGPrices:
    normal: Optional[TCGPrice]
    holofoil: Optional[TCGPrice]
    reverseHolofoil: Optional[TCGPrice]
    firstEditionHolofoil: Optional[TCGPrice]
    firstEditionNormal: Optional[TCGPrice]


@dataclass
class TCGPlayer:
    url: str
    updatedAt: str
    prices: Optional[TCGPrices]



import unittest
from pokemontcgsdkasync.config import __pypi_package_name__, __github_username__, __github_repo_name__


class TestConfig(unittest.TestCase):

    def test_has_proper_package_name(self):
        self.assertEqual('pokemontcgsdkasync', __pypi_package_name__)

    def test_has_proper_github_username(self):
        self.assertEqual('Pole458', __github_username__)

    def test_has_proper_github_repo_name(self):
        self.assertEqual('pokemon-tcg-sdk-python-async', __github_repo_name__)



from dataclasses import dataclass
from typing import Optional

from pokemontcgsdkasync.legality import Legality
from pokemontcgsdkasync.querybuilder import QueryBuilder
from pokemontcgsdkasync.setimage import SetImage


@dataclass
class Set:
    RESOURCE = 'sets'

    id: str
    images: SetImage
    legalities: Legality
    name: str
    printedTotal: int
    ptcgoCode: Optional[str]
    releaseDate: str
    series: str
    total: int
    updatedAt: str

    @staticmethod
    async def find(set_id):
        return await QueryBuilder(Set).find(set_id)

    @staticmethod
    async def where(**kwargs):
        return await QueryBuilder(Set).where(**kwargs)

    @staticmethod
    async def all():
        return await QueryBuilder(Set).all()



import unittest
from typing import List

from pokemontcgsdkasync import Set, AsyncClientContext


class TestSet(unittest.IsolatedAsyncioTestCase):

    async def test_find_returns_set(self):
        async with AsyncClientContext():
            card_set: Set = await Set.find('xy11')
            self.assertEqual('xy11', card_set.id)
            self.assertEqual('Steam Siege', card_set.name)
            self.assertEqual('XY', card_set.series)
            self.assertEqual(114, card_set.printedTotal)
            self.assertEqual(116, card_set.total)
            self.assertEqual('STS', card_set.ptcgoCode)
            self.assertEqual("2016/08/03", card_set.releaseDate)

    async def test_where_filters_on_name(self):
        async with AsyncClientContext():
            sets: List[Set] = await Set.where(q='name:steam')

            self.assertEqual(1, len(sets))
            self.assertEqual('xy11', sets[0].id)

    async def test_all_returns_all_sets(self):
        async with AsyncClientContext():
            sets: List[Set] = await Set.all()

            self.assertGreater(len(sets), 70)



from dataclasses import dataclass
from typing import Optional, List

from pokemontcgsdkasync.ability import Ability
from pokemontcgsdkasync.ancienttrait import AncientTrait
from pokemontcgsdkasync.attack import Attack
from pokemontcgsdkasync.cardimage import CardImage
from pokemontcgsdkasync.legality import Legality
from pokemontcgsdkasync.querybuilder import QueryBuilder
from pokemontcgsdkasync.resistance import Resistance
from pokemontcgsdkasync.set import Set
from pokemontcgsdkasync.tcgplayer import TCGPlayer
from pokemontcgsdkasync.cardmarket import Cardmarket
from pokemontcgsdkasync.weakness import Weakness


@dataclass
class Card:
    RESOURCE = 'cards'

    abilities: Optional[List[Ability]]
    artist: Optional[str]
    ancientTrait: Optional[AncientTrait]
    attacks: Optional[List[Attack]]
    cardmarket: Optional[Cardmarket]
    convertedRetreatCost: Optional[int]
    evolvesFrom: Optional[str]
    flavorText: Optional[str]
    hp: Optional[str]
    id: str
    images: CardImage
    legalities: Legality
    regulationMark: Optional[str]
    name: str
    nationalPokedexNumbers: Optional[List[int]]
    number: str
    rarity: Optional[str]
    regulationMark: Optional[str]
    resistances: Optional[List[Resistance]]
    retreatCost: Optional[List[str]]
    rules: Optional[List[str]]
    set: Set
    subtypes: Optional[List[str]]
    supertype: str
    tcgplayer: Optional[TCGPlayer]
    types: Optional[List[str]]
    weaknesses: Optional[List[Weakness]]

    @staticmethod
    async def find(card_id: str):
        return await QueryBuilder(Card, Card.transform).find(card_id)

    @staticmethod
    async def where(**kwargs):
        return await QueryBuilder(Card, Card.transform).where(**kwargs)

    @staticmethod
    async def all():
        return await QueryBuilder(Card, Card.transform).all()

    @staticmethod
    def transform(response):
        if response.get('tcgplayer', {}).get('prices', {}).get('1stEditionNormal'):
            response['tcgplayer']['prices']['firstEditionNormal'] = response['tcgplayer']['prices'].pop(
                '1stEditionNormal')
        if response.get('tcgplayer', {}).get('prices', {}).get('1stEditionHolofoil'):
            response['tcgplayer']['prices']['firstEditionHolofoil'] = response['tcgplayer']['prices'].pop(
                '1stEditionHolofoil')
        return response



from dataclasses import dataclass
from typing import Optional, List


@dataclass
class Attack:
    name: str
    cost: List[str]
    convertedEnergyCost: int
    damage: Optional[str]
    text: Optional[str]



import unittest
from typing import List

from pokemontcgsdkasync import Supertype, AsyncClientContext


class TestSupertype(unittest.IsolatedAsyncioTestCase):

    async def test_all_returns_supertypes(self):
        async with AsyncClientContext():
            supertypes: List[Supertype] = await Supertype.all()

            self.assertEqual(["Energy", "Pokémon", "Trainer"], supertypes)



import pathlib

from setuptools import setup, find_packages

from pokemontcgsdkasync.config import __version__, __pypi_package_name__, __github_username__, __github_repo_name__

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text(encoding='utf-8')

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'sphinx',
]

url = 'https://github.com/' + __github_username__ + '/' + __github_repo_name__
download_url = "{}/tarball/{}".format(url, __version__)

setup(
    name=__pypi_package_name__,
    version=__version__,
    description='Pokemon TCG SDK for pokemontcg.io using asyncio',
    long_description=README,
    long_description_content_type="text/markdown",
    url=url,
    author='Paolo D\'Alessandro',
    author_email='pole.gamedev@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    keywords='pokemon tcg sdk trading card game api rest async',
    download_url=download_url,
    packages=find_packages(),
    include_package_data=False,
    python_requires=">=3.6",
    install_requires=[
        # add your dependencies here
        # remember to use 'package-name>=x.y.z,<x.y+1.0' notation (this way you get bugfixes)
        'dacite>=1.6.0,<2.0.0',
        'aiohttp>=3.8.1,<4.0.0',
    ],
    extras_require={
        'tests': tests_require,
    },
    # entry_points={
    #     'console_scripts': [
    #         # add cli scripts here in this form:
    #         # 'pokemontcgsdk=pokemontcgsdk.cli:main',
    #     ],
    # },
)



from dataclasses import dataclass
from typing import Optional


@dataclass
class CardmarketPrices:
    averageSellPrice: Optional[float]
    lowPrice: Optional[float]
    trendPrice: Optional[float]
    germanProLow: Optional[float]
    suggestedPrice: Optional[float]
    reverseHoloSell: Optional[float]
    reverseHoloLow: Optional[float]
    reverseHoloTrend: Optional[float]
    lowPriceExPlus: Optional[float]
    avg1: Optional[float]
    avg7: Optional[float]
    avg30: Optional[float]
    reverseHoloAvg1: Optional[float]
    reverseHoloAvg7: Optional[float]
    reverseHoloAvg30: Optional[float]


@dataclass
class Cardmarket:
    url: str
    updatedAt: str
    prices: Optional[CardmarketPrices]
